@@ -5,7 +5,7 @@ import asyncio
 import os
 from contextlib import suppress
 from pathlib import Path
-from typing import Optional
+from typing import Any, Dict, Optional, Type
 
 import pandas as pd
 from aiohttp import ClientSession
@@ -18,6 +18,7 @@ from toucan_connectors.oauth2_connector.oauth2connector import (
 )
 from toucan_connectors.toucan_connector import (
     ConnectorSecretsForm,
+    DataSlice,
     ToucanConnector,
     ToucanDataSource,
     strlist_to_enum,
@@ -44,6 +45,10 @@ class GoogleSheets2DataSource(ToucanDataSource):
     - header_row
     """
 
+    domain: str = Field(
+        ...,
+        title='dataset',
+    )
     spreadsheet_id: str = Field(
         ...,
         title='ID of the spreadsheet',
@@ -56,6 +61,18 @@ class GoogleSheets2DataSource(ToucanDataSource):
     header_row: int = Field(
         0, title='Header row', description='Row of the header of the spreadsheet'
     )
+    rows_limit: int = Field(
+        None, title='Rows limit', description='Maximum number of rows to retrieve'
+    )
+    parameters: dict = Field(None, description='Additional URL parameters')
+
+    class Config:
+        @staticmethod
+        def schema_extra(schema: Dict[str, Any], model: Type['GoogleSheets2DataSource']) -> None:
+            keys = schema['properties'].keys()
+            prio_keys = ['domain', 'spreadsheet_id', 'sheet']
+            new_keys = prio_keys + [k for k in keys if k not in prio_keys]
+            schema['properties'] = {k: schema['properties'][k] for k in new_keys}
 
     @classmethod
     def get_form(cls, connector: 'GoogleSheets2Connector', current_config, **kwargs):
@@ -149,8 +166,13 @@ class GoogleSheets2Connector(ToucanConnector):
             available_sheets = [str(x['properties']['title']) for x in data['sheets']]
             data_source.sheet = available_sheets[0]
 
+        if data_source.rows_limit:
+            ranges = f'!1:{data_source.rows_limit+1}'
+        else:
+            ranges = ''
+
         # https://developers.google.com/sheets/api/samples/reading
-        read_sheet_endpoint = f'{data_source.spreadsheet_id}/values/{data_source.sheet}?valueRenderOption=UNFORMATTED_VALUE'
+        read_sheet_endpoint = f'{data_source.spreadsheet_id}/values/{data_source.sheet}{ranges}?valueRenderOption=UNFORMATTED_VALUE'
         full_url = f'{self._baseroute}{read_sheet_endpoint}'
         # Rajouter le param FORMATTED_VALUE pour le séparateur de décimal dans la Baseroute
         data = self._run_fetch(full_url)['values']
@@ -189,3 +211,32 @@ class GoogleSheets2Connector(ToucanConnector):
             return ConnectorStatus(status=True, message=f"Connected as {user_info.get('email')}")
         except HttpError:
             return ConnectorStatus(status=False, error="Couldn't retrieve user infos")
+
+    def get_slice(
+        self,
+        data_source: GoogleSheets2DataSource,
+        permissions: Optional[dict] = None,
+        offset: int = 0,
+        limit=50,
+    ) -> DataSlice:
+        """
+        Method to retrieve a part of the data as a pandas dataframe
+        and the total size filtered with permissions
+
+        - offset is the index of the starting row
+        - limit is the number of pages to retrieve
+        Exemple: if offset = 5 and limit = 10 then 10 results are expected from 6th row
+        """
+        preview_datasource = GoogleSheets2DataSource(
+            domain=data_source.domain,
+            name=data_source.name,
+            spreadsheet_id=data_source.spreadsheet_id,
+            sheet=data_source.sheet,
+            header_row=data_source.header_row,
+            rows_limit=limit,
+        )
+        df = self.get_df(preview_datasource, permissions)
+        if limit is not None:
+            return DataSlice(df[offset : offset + limit], len(df))
+        else:
+            return DataSlice(df[offset:], len(df))
